@@ -31,6 +31,9 @@ import base64
 import gtk
 from sgmllib import SGMLParser
 from threading import Thread
+from xml.dom.minidom import Document
+from xml.dom import minidom
+import xmlrpclib
 
 # http://diveintomark.org/archives/2002/05/31/rss_autodiscovery_in_python
 def getRSSLink(url):
@@ -121,6 +124,103 @@ class FeedManager(gobject.GObject):
         except IOError:
             pass
 
+    def export_opml(self, file_path, callback, data=None):
+
+        # Create a new thread to update from network
+        exporting_thread = Thread(target=self._export_opml_in_thread,
+                               args=(file_path, callback, data))
+        try:
+            exporting_thread.start()
+        except IOError:
+            callback(False)
+
+    def _export_opml_in_thread(self, file_path, callback, user_data):
+        # Create the minidom document
+        opml_doc = Document()
+
+        # Add tags and create the body of the document
+        root = opml_doc.createElement('opml')
+        root.setAttribute('version','1.0')
+        root.appendChild(opml_doc.createElement('head'))
+        body = opml_doc.createElement('body')
+        for feed_data in self.feed_data_list:
+            outline = opml_doc.createElement('outline')
+            outline.setAttribute('title',feed_data.feed.title)
+            outline.setAttribute('text',feed_data.feed.subtitle)
+            outline.setAttribute('xmlUrl',feed_data['href'])
+            body.appendChild(outline)
+
+        root.appendChild(body)
+        opml_doc.appendChild(root)
+
+        try:
+            retval = True
+            file_opml = open(file_path,"w")
+            opml_doc.writexml(file_opml, "    ", "", "\n", "UTF-8")
+        except IOError :
+            print 'Error exporting OPML, could not write to ' + file_path
+            retval = False
+
+        gtk.gdk.threads_enter()
+        callback(retval)
+        gtk.gdk.threads_leave()
+
+    def import_opml(self, file_path, callback, data=None):
+
+        # Create a new thread to update from network
+        importing_thread = Thread(target=self._import_opml_in_thread,
+                               args=(file_path, callback, data))
+        try:
+            importing_thread.start()
+        except IOError:
+            callback(None, None)
+
+    def _import_opml_in_thread(self, file_path, callback, user_data):
+        if (not (os.path.exists(file_path) and os.path.isfile(file_path)) or
+            (os.path.splitext(file_path)[1] != '.opml')):
+            gtk.gdk.threads_enter()
+            callback(None,None)
+            gtk.gdk.threads_leave()
+            return
+
+        try:
+            doc=open(file_path,'r')
+            opml_doc = minidom.parse(doc)
+            opml_doc.getElementsByTagName('outline')
+            for node in opml_doc.getElementsByTagName('outline'):
+                if node.getAttribute('xmlUrl') != '':
+                    url = node.attributes['xmlUrl'].value
+                    # Add feed to manager
+                    self._add_feed_in_thread(url, callback, user_data)
+        except IOError :
+            gtk.gdk.threads_enter()
+            callback(None,None)
+            gtk.gdk.threads_leave()
+            return
+
+        doc.close()
+
+    def find_feed(self, key_words, dialog, callback):
+        # Create a new thread to search
+        find_thread = Thread(target=self._find_feed_in_thread,
+                               args=(key_words, dialog, callback))
+
+        return find_thread.start()
+
+    def _find_feed_in_thread(self, key_words, dialog, callback):
+        try:
+            server = xmlrpclib.Server('http://www.syndic8.com/xmlrpc.php')
+            feedids = server.syndic8.FindFeeds(key_words,'last_pubdate',25,0)
+            infolist = server.syndic8.GetFeedInfo(feedids, ['imageurl','sitename','dataurl'])
+        except:
+            infolist = None
+            print 'Error while accessing syndic8.com'
+
+        if callback:
+            gtk.gdk.threads_enter()
+            callback(key_words, dialog, infolist)
+            gtk.gdk.threads_leave()
+
     def save(self, callback, data=None):
         # TODO: migrate to "with" statement when available
         try:
@@ -243,7 +343,8 @@ class FeedManager(gobject.GObject):
             updated_feed_data = ReSiStanceFeedDict(feedparser.parse(feed_data.href))
 
             # In case of network failure
-            if updated_feed_data == None or updated_feed_data.entries == None:
+            if updated_feed_data == None or updated_feed_data.entries == None or \
+                    len(updated_feed_data.entries) == 0:
                 continue
 
             updated_feed_date = updated_feed_data.feed.get('updated_parsed') or \
