@@ -20,7 +20,7 @@
 #    along with ReSiStance.  If not, see <http://www.gnu.org/licenses/>.
 #########################################################################
 
-import pango
+from pango import ELLIPSIZE_END
 import feedparser
 import locale
 import gettext
@@ -29,7 +29,7 @@ import hildon
 import pygtk
 pygtk.require('2.0')
 import gtk
-import webkit
+from webkit import WebView
 import os
 import urllib2
 from portrait import FremantleRotation
@@ -77,12 +77,12 @@ def unescape(text):
 
 def get_author_from_item(item):
     words = item.title.rsplit(":")
-    if len(words) > 1:
-        author = words[0].lstrip()
-    elif 'publisher_detail' in item:
+    if 'publisher_detail' in item:
         author = item.publisher_detail.name
     elif 'author' in item:
         author = item.author
+    elif len(words) > 1:
+        author = words[0].lstrip()
     else:
         author = ''
 
@@ -101,7 +101,7 @@ class FeedsView(hildon.GtkTreeView):
         super(FeedsView, self).__init__(ui_mode)
 
         text_renderer = gtk.CellRendererText()
-        text_renderer.set_property('ellipsize', pango.ELLIPSIZE_END)
+        text_renderer.set_property('ellipsize', ELLIPSIZE_END)
 
         pix_renderer = gtk.CellRendererPixbuf()
 
@@ -114,25 +114,9 @@ class FeedsView(hildon.GtkTreeView):
         self.append_column(column);
 
         date_renderer = gtk.CellRendererText()
-        column = gtk.TreeViewColumn('Date', date_renderer)
-        column.set_cell_data_func(date_renderer, self._unread_view_data_func)
+        column = gtk.TreeViewColumn('Date', date_renderer, markup = self.FEEDS_READ_COLUMN)
         self.append_column(column);
 
-
-    def _unread_view_data_func(self, column, renderer, model, feed_iter):
-        feed_data, unread_count = model.get(feed_iter, self.FEEDS_DATA_COLUMN,
-                                      self.FEEDS_READ_COLUMN)
-
-        if feed_data.status == self.DUMMY_FEED_STATUS:
-            renderer.set_property('text', '')
-            return
-
-        if unread_count:
-            renderer.set_property('markup', '<span size="xx-small">%d '
-                                  '%s\n%s</span>' % (unread_count, _('unread'), _('items')))
-        else:
-            renderer.set_property('markup', '')
-
     def add_dummy_feed(self, url):
 
         store = self.get_model()
@@ -145,19 +129,22 @@ class FeedsView(hildon.GtkTreeView):
         store.set(feed_iter, self.FEEDS_ICON_COLUMN, None,
                   self.FEEDS_NAME_COLUMN, self.get_feed_title_markup(dummy_feed_data),
                   self.FEEDS_DATA_COLUMN, dummy_feed_data,
-                  self.FEEDS_READ_COLUMN, 0)
+                  self.FEEDS_READ_COLUMN, '')
 
         return (dummy_feed_data,feed_iter)
 
     def get_feed_title_markup(self, feed_data):
-        if 'link' in  feed_data.feed:
-            url = feed_data.feed.link
+        if 'subtitle' in feed_data.feed:
+            secondary = feed_data.feed.subtitle
         else:
-            url = feed_data.href
+            if 'link' in  feed_data.feed:
+                secondary = feed_data.feed.link
+            else:
+                secondary = feed_data.href
 
         return '%s\n<span face="monospace" foreground="%s"' \
             'size="x-small">%s</span>' % \
-            (feed_data.feed.title, secondary_color.to_string(), url)
+            (feed_data.feed.title, secondary_color.to_string(), secondary)
 
     def sort(self, order):
         if (order == constants.ASCENDING_ORDER):
@@ -238,6 +225,33 @@ class NewFeedDialog(gtk.Dialog):
         else:
             self.set_response_sensitive(gtk.RESPONSE_ACCEPT, False)
 
+class FindFeedsDialog(gtk.Dialog):
+    def __init__(self, parent):
+        super(FindFeedsDialog, self).__init__(_('Find Feeds'), parent,
+                                            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
+                                            (_('Find'), gtk.RESPONSE_ACCEPT))
+
+
+        self.entry = hildon.Entry(gtk.HILDON_SIZE_FINGER_HEIGHT)
+        self.entry.set_input_mode(gtk.HILDON_GTK_INPUT_MODE_FULL)
+        self.entry.connect('changed', self._entry_changed_cb)
+
+        caption = gtk.HBox(False, 16)
+        caption.pack_start(gtk.Label(_('Keywords') + ':'), False, False)
+        caption.pack_start(self.entry)
+        self.vbox.add(caption)
+
+        self.set_response_sensitive(gtk.RESPONSE_ACCEPT, False)
+
+    def _entry_changed_cb(self, entry):
+        if len(entry.get_text()) > 0:
+            self.set_response_sensitive(gtk.RESPONSE_ACCEPT, True)
+        else:
+            self.set_response_sensitive(gtk.RESPONSE_ACCEPT, False)
+
+    def _delete_event_cb(self, widget, event):
+        return True #stop the signal propagation
+
 class FeedsWindow(hildon.StackableWindow):
 
     def __init__(self):
@@ -265,7 +279,7 @@ class FeedsWindow(hildon.StackableWindow):
 
         # Feeds
         self.view = FeedsView(gtk.HILDON_UI_MODE_NORMAL)
-        self.view.set_model (gtk.ListStore(gtk.gdk.Pixbuf, str, gobject.TYPE_PYOBJECT, int))
+        self.view.set_model (gtk.ListStore(gtk.gdk.Pixbuf, str, gobject.TYPE_PYOBJECT, str))
         self.view.connect ("row-activated", self._on_feed_activated)
         self.view.show()
 
@@ -312,7 +326,6 @@ class FeedsWindow(hildon.StackableWindow):
         except IOError:
             pass
 
-
     def _on_feed_activated(self, treeview, path, column):
         feed_iter = self.view.get_model().get_iter(path)
         feed_data = self.view.get_model().get_value(feed_iter, self.view.FEEDS_DATA_COLUMN)
@@ -336,7 +349,8 @@ class FeedsWindow(hildon.StackableWindow):
 
         # Reset unread cache (the value have most likely changed).
         unread_count = len([entry for entry in feed_data.entries if entry.read == False])
-        self.view.get_model().set(feed_iter, self.view.FEEDS_READ_COLUMN, unread_count)
+        self.view.get_model().set(feed_iter, self.view.FEEDS_READ_COLUMN,
+                                  self._get_visual_unread_text(unread_count))
 
     def _on_configuration_changed(self, window, event):
         # Change alignment padding. We don't want padding in portrait
@@ -392,6 +406,22 @@ class FeedsWindow(hildon.StackableWindow):
         button.connect('clicked', self._preferences_cb)
         menu.append(button)
 
+        button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
+        button.set_label(_('Export Feeds'))
+        button.connect('clicked', self._export_feed_cb)
+        menu.append(button)
+        self.export_opml_button = button
+
+        button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
+        button.set_label(_('Import Feeds'))
+        button.connect('clicked', self._import_feed_cb)
+        menu.append(button)
+
+        button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
+        button.set_label(_('Find Feeds'))
+        button.connect('clicked', self._find_feed_cb)
+        menu.append(button)
+
         menu.show_all()
         self.set_app_menu(menu)
 
@@ -430,6 +460,9 @@ class FeedsWindow(hildon.StackableWindow):
         for path in paths:
             try:
                 manager.remove_feed(model[path][FeedsView.FEEDS_DATA_COLUMN])
+                feed_data_list = manager.get_feed_list()
+                if len(feed_data_list) == 0:
+                    self.export_opml_button.hide()
             except IOError:
                 print 'Could not remove', constants.RSS_DB_FILE
                 hildon.hildon_banner_show_information(self, '', _('File error while removing Feed'))
@@ -463,6 +496,13 @@ class FeedsWindow(hildon.StackableWindow):
         # Ask manager to update feed
         manager.update_all(self._all_feeds_updated_cb, None)
 
+    def _get_visual_unread_text(self, unread_count):
+        if unread_count:
+            return '<span size="xx-small">%d ' \
+                '%s\n%s</span>' % (unread_count, _('unread'), _('items'))
+        else:
+            return ''
+
     def _all_feeds_updated_cb(self, data):
         hildon.hildon_gtk_window_set_progress_indicator(self, False)
 
@@ -470,7 +510,7 @@ class FeedsWindow(hildon.StackableWindow):
         store = self.view.get_model()
         for row in store:
             unread_count = len([entry for entry in row[self.view.FEEDS_DATA_COLUMN].entries if entry.read == False])
-            row[self.view.FEEDS_READ_COLUMN] = unread_count
+            row[self.view.FEEDS_READ_COLUMN] = self._get_visual_unread_text(unread_count)
 
     def _new_feed_response_cb(self, dialog, response):
 
@@ -491,18 +531,27 @@ class FeedsWindow(hildon.StackableWindow):
 
             dialog.destroy()
 
-    def _feed_added_cb(self, pixbuf, new_feed_data, row_reference):
+    def _feed_added_cb(self, pixbuf, new_feed_data, row_reference=None):
+
+        feed_data_list = manager.get_feed_list()
+        if len(feed_data_list) > 0:
+            self.export_opml_button.show()
 
         # Remove progress information
         hildon.hildon_gtk_window_set_progress_indicator(self, False)
 
         store = self.view.get_model()
-        feed_iter = store.get_iter(row_reference.get_path())
+        if row_reference != None:
+            feed_iter = store.get_iter(row_reference.get_path())
+        else:
+            feed_iter = store.append()
 
         # Check that the feed data was retrieved
         if new_feed_data == None:
+            message = _('Could not retrieve feed')
             dummy_data = store.get_value (feed_iter, self.view.FEEDS_DATA_COLUMN)
-            message = _('Cannot retrieve feed from') + ' ' + dummy_data.href
+            if dummy_data:
+                message += 'from ' + dummy_data.href
             store.remove(feed_iter)
             hildon.hildon_banner_show_information(self, '', message)
             return
@@ -511,7 +560,7 @@ class FeedsWindow(hildon.StackableWindow):
         store.set(feed_iter, self.view.FEEDS_ICON_COLUMN, pixbuf,
                   self.view.FEEDS_NAME_COLUMN, self.view.get_feed_title_markup(new_feed_data),
                   self.view.FEEDS_DATA_COLUMN, new_feed_data,
-                  self.view.FEEDS_READ_COLUMN, len(new_feed_data.entries))
+                  self.view.FEEDS_READ_COLUMN, self._get_visual_unread_text(len(new_feed_data.entries)))
 
         # Take screenshot
         hildon.hildon_gtk_window_take_screenshot(self, False)
@@ -528,10 +577,49 @@ class FeedsWindow(hildon.StackableWindow):
             # Update rotation
             self.rotation_object.set_mode(settings.rotation_mode)
 
+    def _feed_exported_cb(self, exported):
+        if exported:
+            message = _('Feeds exported')
+        else:
+            message = _('Error exporting feeds')
+        hildon.hildon_banner_show_information(self, '', message)
+        # Remove progress information
+        hildon.hildon_gtk_window_set_progress_indicator(self, False)
+
+    def _initial_dialog_new_cb(self, button, dialog):
+        dialog.destroy()
+        self._new_feed_cb(button)
+
+    def _initial_dialog_find_cb(self, button, dialog):
+        dialog.destroy()
+        self._find_feed_cb(button)
+
+    def _show_initial_dialog(self):
+        dialog = gtk.Dialog()
+        dialog.set_title(_('Start adding feeds'))
+
+        # Create buttons
+        add_feed_button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
+        add_feed_button.set_label(_('New feed'))
+        add_feed_button.connect('clicked', self._initial_dialog_new_cb, dialog)
+
+        find_feed_button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
+        find_feed_button.set_label(_('Find Feeds'))
+        find_feed_button.connect('clicked', self._initial_dialog_find_cb, dialog)
+
+        dialog.vbox.add(add_feed_button)
+        dialog.vbox.add(find_feed_button)
+
+        add_feed_button.show()
+        find_feed_button.show()
+        dialog.show()
+
     def _feed_data_loaded(self, user_data):
         # TODO: migrate to "with" statement when available
         try:
             feed_data_list = manager.get_feed_list()
+            if len(feed_data_list) == 0:
+                self.export_opml_button.hide()
         except IOError:
             print 'cannot open', constants.RSS_DB_FILE
         else:
@@ -544,16 +632,43 @@ class FeedsWindow(hildon.StackableWindow):
                 store.set(feed_iter, self.view.FEEDS_ICON_COLUMN, pixbuf,
                           self.view.FEEDS_NAME_COLUMN, self.view.get_feed_title_markup(feed_data),
                           self.view.FEEDS_DATA_COLUMN, feed_data,
-                          self.view.FEEDS_READ_COLUMN, unread_count)
+                          self.view.FEEDS_READ_COLUMN, self._get_visual_unread_text(unread_count))
                 # Improve responsiveness
 #                while gtk.events_pending():
 #                    gtk.main_iteration_do()
 
-        # If there is no data then show the new feed dialog
+        # If there is no data then show a dialog
         if len(feed_data_list) == 0:
-            dialog = NewFeedDialog(self)
-            dialog.connect('response', self._new_feed_response_cb)
-            dialog.show_all()
+            self._show_initial_dialog()
+
+    def _find_feed_response_cb(self, dialog, response):
+        if response == gtk.RESPONSE_ACCEPT:
+            key_words = dialog.entry.get_text()
+
+            # Quickly show feedback to user
+            hildon.hildon_gtk_window_set_progress_indicator(dialog, True)
+
+            manager.find_feed(key_words, dialog, self._found_cb)
+            dialog.set_response_sensitive(gtk.RESPONSE_ACCEPT, False)
+
+            # Disable canceling
+            dialog.connect('delete-event', lambda widget,event : True)
+
+    def _found_cb(self, key_words, dialog, feeds_info):
+        if feeds_info == None:
+            # Something went wrong
+            hildon.hildon_banner_show_information(self, '', _('Error finding feeds. Server error.'))
+            dialog.destroy()
+            return
+
+        news_window = FindWindow(feeds_info, key_words, self)
+        hildon.Program.get_instance().add_window(news_window)
+        news_window.show()
+
+        dialog.set_response_sensitive(gtk.RESPONSE_ACCEPT, True)
+        hildon.hildon_gtk_window_set_progress_indicator(dialog, False)
+
+        dialog.destroy()
 
     def _sort_ascending_cb(self, button):
         settings.feeds_order = constants.ASCENDING_ORDER
@@ -563,6 +678,59 @@ class FeedsWindow(hildon.StackableWindow):
         settings.feeds_order = constants.DESCENDING_ORDER
         self.view.sort(constants.DESCENDING_ORDER)
 
+    def _export_feed_cb(self, button):
+        stack = hildon.WindowStack.get_default()
+        window = stack.peek()
+
+        chooser = hildon.FileChooserDialog(window, gtk.FILE_CHOOSER_ACTION_SAVE)
+        chooser.set_transient_for(window)
+        # Don't like this at all. Isn't there any define in the platform ?
+        chooser.set_current_folder('/home/user/MyDocs/.documents')
+        chooser.set_current_name('resistance-feeds')
+
+        #Overwrite the file if already exists
+        chooser.set_do_overwrite_confirmation(True)
+        chooser.set_default_response(gtk.RESPONSE_OK)
+
+        response = chooser.run()
+        if response == gtk.RESPONSE_OK:
+            hildon.hildon_gtk_window_set_progress_indicator(self, True)
+            manager.export_opml(chooser.get_filename() + '.opml', self._feed_exported_cb)
+        elif response == gtk.RESPONSE_CANCEL:
+            print 'Closed, no files selected'
+        chooser.destroy()
+
+    def _import_feed_cb(self, button):
+        stack = hildon.WindowStack.get_default()
+        window = stack.peek()
+
+        #Calling a file chooser
+        chooser = hildon.FileChooserDialog(window ,gtk.FILE_CHOOSER_ACTION_OPEN)
+        chooser.set_transient_for(window)
+        # Don't like this at all. Isn't there any define in the platform ?
+        chooser.set_current_folder('/home/user/MyDocs/.documents')
+        chooser.set_default_response(gtk.RESPONSE_OK)
+
+        #Filter for opml files
+        filter = gtk.FileFilter()
+        filter.set_name("OPML")
+        filter.add_pattern("*.opml")
+        chooser.add_filter(filter)
+
+        response = chooser.run()
+        if response == gtk.RESPONSE_OK:
+            hildon.hildon_gtk_window_set_progress_indicator(self, True)
+            manager.import_opml(chooser.get_filename(), self._feed_added_cb)
+        elif response == gtk.RESPONSE_CANCEL:
+            print 'Closed, no files selected'
+
+        chooser.destroy()
+
+    def _find_feed_cb(self, button):
+        dialog = FindFeedsDialog(self)
+        dialog.connect('response', self._find_feed_response_cb)
+        dialog.show_all()
+
 class EntriesView(hildon.GtkTreeView):
 
     ENTRIES_ICON_COLUMN = 0
@@ -581,9 +749,8 @@ class EntriesView(hildon.GtkTreeView):
         self.append_column(column);
 
         text_renderer = gtk.CellRendererText()
-        text_renderer.set_property('ellipsize', pango.ELLIPSIZE_END)
-        column = gtk.TreeViewColumn('Name', text_renderer)
-        column.set_cell_data_func(text_renderer, self._view_data_func)
+        text_renderer.set_property('ellipsize', ELLIPSIZE_END)
+        column = gtk.TreeViewColumn('Name', text_renderer, markup = 1)
         column.set_expand(True)
         self.append_column(column);
 
@@ -592,8 +759,7 @@ class EntriesView(hildon.GtkTreeView):
         column.set_expand(False)
         self.append_column(column);
 
-    def _view_data_func(self, column, renderer, model, iter):
-        entry = model.get_value(iter, self.ENTRIES_DATA_COLUMN)
+    def get_visual_entry_text(self, entry):
 
         # TODO: show entry.summary ?
         author = get_author_from_item(entry)
@@ -601,7 +767,7 @@ class EntriesView(hildon.GtkTreeView):
             author = self.fallback_author
 
         words = entry.title.rsplit(":")
-        if len(words) > 1:
+        if len(words) > 1 and author == words[0].lstrip():
             title = words[1].lstrip()
         else:
             title = entry.title
@@ -611,17 +777,18 @@ class EntriesView(hildon.GtkTreeView):
         else:
             color = active_color.to_string()
 
-        renderer.set_property('markup',
-                              '<span foreground="%s">%s</span>\n<span size="small">%s</span>'
-                              % (color,
-                                 gobject.markup_escape_text(author),
-                                 gobject.markup_escape_text(title)))
+        return '<span foreground="' + color + '">' \
+            + gobject.markup_escape_text(author) \
+            + '</span>\n<span size="small">' \
+            + gobject.markup_escape_text(title) + '</span>'
 
     def get_visual_entry_date(self, entry):
-
         if entry.has_key('updated_parsed') == False:
-            return '<span size="xx-small">%s</span>' % \
-                entry.updated
+            if entry.has_key('updated'):
+                return '<span size="xx-small">%s</span>' % \
+                    entry.updated
+            else:
+                return ''
 
         now = time.localtime()
 
@@ -661,7 +828,9 @@ class EntriesWindow(hildon.StackableWindow):
 
         for entry in self.feed_data.entries:
             entry_iter = model.append()
-            model.set(entry_iter, 1, entry.title, 2, entry,
+            model.set(entry_iter,
+                      1, self.view.get_visual_entry_text(entry),
+                      2, entry,
                       3, self.view.get_visual_entry_date(entry))
 
         self.align = gtk.Alignment(0, 0, 1, 1)
@@ -701,10 +870,20 @@ class EntriesWindow(hildon.StackableWindow):
             entry['read'] = True
 
     def _on_entry_activated(self, treeview, path, column):
-        item_window = ItemWindow(self.feed_data, treeview.get_model(), path)
+        item_window = ItemWindow(self.feed_data)
         hildon.Program.get_instance().add_window(item_window)
+
+        item_window.connect('item-read', self._item_read_cb)
+        item_window.set_item_from_model(treeview.get_model(), path)
         item_window.show()
 
+    def _item_read_cb(self, item_window, path):
+        entry = self.view.get_model()[path][2]
+
+        # Set item as read and update the UI
+        entry['read'] = True
+        self.view.get_model()[path][1] = self.view.get_visual_entry_text(entry)
+
     def _update_feed_cb(self, button):
         url = self.feed_data.href
 
@@ -724,7 +903,8 @@ class EntriesWindow(hildon.StackableWindow):
         model.clear()
         for entry in self.feed_data.entries:
             entry_iter = model.append()
-            model.set(entry_iter, 1, entry.title, 2, entry,
+            model.set(entry_iter, 1, self.view.get_visual_entry_text(entry),
+                      2, entry,
                       3, self.view.get_visual_entry_date(entry))
 
         return
@@ -739,11 +919,9 @@ class EntriesWindow(hildon.StackableWindow):
 
 class ItemWindowHeader(gtk.HBox):
 
-    def __init__(self, homogeneous, spacing, item):
+    def __init__(self, homogeneous, spacing):
         super(ItemWindowHeader, self).__init__(homogeneous, spacing)
 
-        self.item = item
-
         # Header
         header_vbox = gtk.VBox()
         navigation_hbox = gtk.HBox(False, 0)
@@ -751,10 +929,9 @@ class ItemWindowHeader(gtk.HBox):
         # Author and URL
         self.title = gtk.Label('')
         self.title.set_alignment(0,0.5)
-        self.title.set_ellipsize(pango.ELLIPSIZE_END)
+        self.title.set_ellipsize(ELLIPSIZE_END)
         header_vbox.pack_start(self.title, True, False)
         self.title.show()
-        self.set_item(item)
         header_vbox.show()
 
         # Navigation buttons. Sizes are important. We set the size of the icon but
@@ -787,31 +964,26 @@ class ItemWindowHeader(gtk.HBox):
 
         self.item = item
 
-        # Mark item as read
-        item['read'] = True
-
         self.title.set_markup('<span size="small">' + unescape(item.title) + '</span>')
         #self.url.set_uri(item.link)
 
 class ItemWindow(hildon.StackableWindow):
+    __gsignals__ = {
+        "item-read": (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT, )),
+        }
 
-    def __init__(self, feed_data, model, path):
+    def __init__(self, feed_data):
         super(ItemWindow, self).__init__()
 
-        item_iter = model.get_iter(path)
-        item = model.get_value(item_iter, EntriesView.ENTRIES_DATA_COLUMN)
-
         self.feed_data = feed_data
-        self.model = model
-        self.model_iter = model.get_iter(path)
 
         # Header
-        self.item_window_header = ItemWindowHeader(False, 16, item)
+        self.item_window_header = ItemWindowHeader(False, 16)
         self.item_window_header.button_up.connect("clicked", self._up_button_clicked)
         self.item_window_header.button_down.connect("clicked", self._down_button_clicked)
 
         # HTML renderer
-        self.view = webkit.WebView()
+        self.view = WebView()
         self.view.set_full_content_zoom(True)
 
         # Disable text selection
@@ -849,7 +1021,6 @@ class ItemWindow(hildon.StackableWindow):
         self._create_toolbar()
 
         self.connect('configure-event', self._on_configuration_changed)
-        self.set_item(item)
 
     def _on_configuration_changed(self, window, event):
 
@@ -979,6 +1150,17 @@ class ItemWindow(hildon.StackableWindow):
         item = self.model.get_value(self.model_iter, EntriesView.ENTRIES_DATA_COLUMN)
         self.set_item(item)
 
+    def set_item_from_model(self, model, path):
+
+        self.model = model
+        self.model_iter = model.get_iter(path)
+
+        item_iter = model.get_iter(path)
+        item = model.get_value(item_iter, EntriesView.ENTRIES_DATA_COLUMN)
+
+        self.item_window_header.set_item(item)
+        self.set_item(item)
+
     def set_item(self, item):
 
         self.set_title(get_author_from_item(item))
@@ -1020,3 +1202,105 @@ class ItemWindow(hildon.StackableWindow):
             toolbar_button_list[0].set_sensitive(True)
             toolbar_button_list[1].set_sensitive(True)
 
+        # Mark item as read
+        item['read'] = True
+
+        # Emit item read
+        self.emit('item-read', self.model.get_path(self.model_iter))
+
+class FindView(hildon.GtkTreeView):
+
+    FIND_DATA_COLUMN = 0
+    FIND_NAME_COLUMN = 1
+
+    def __init__(self, ui_mode):
+        super(FindView, self).__init__(ui_mode)
+
+        # Add columns
+        text_renderer = gtk.CellRendererText()
+        text_renderer.set_property('ellipsize', ELLIPSIZE_END)
+        column = gtk.TreeViewColumn('Name', text_renderer, markup=1)
+        column.set_expand(True)
+        self.append_column(column)
+
+
+class FindWindow(hildon.StackableWindow):
+
+    def __init__(self, feedsinfo, key_words, feedsWindow):
+        super(FindWindow, self).__init__()
+
+        self.feeds_info = feedsinfo
+
+        self.view = FindView(gtk.HILDON_UI_MODE_EDIT)
+        self.view.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
+        self.view.get_selection().unselect_all()
+
+        self.edit_toolbar = hildon.EditToolbar()
+
+        self.edit_toolbar_button_handler = 0
+        self.edit_toolbar.set_label(_('Select Feeds to add'))
+        self.edit_toolbar.set_button_label(_('Add'))
+        self.edit_toolbar.connect('arrow-clicked', self._restore_normal_mode)
+
+        if self.edit_toolbar.handler_is_connected (self.edit_toolbar_button_handler):
+            self.edit_toolbar.disconnect(self.edit_toolbar_button_handler)
+        self.edit_toolbar_button_handler = \
+            self.edit_toolbar.connect('button-clicked', self._add_button_clicked_cb, feedsWindow)
+
+        self.set_edit_toolbar(self.edit_toolbar)
+        self.edit_toolbar.show()
+        self.fullscreen()
+
+        self.view.set_model (gtk.ListStore(str, str))
+
+        store = self.view.get_model()
+
+        for feed_info in self.feeds_info:
+            feed_iter = store.append()
+
+            store.set(feed_iter, self.view.FIND_NAME_COLUMN, gobject.markup_escape_text(feed_info['sitename']),
+                      self.view.FIND_DATA_COLUMN, feed_info['dataurl'])
+
+        self.view.show()
+
+        self.align = gtk.Alignment(0, 0, 1, 1)
+        self.align.set_padding(4, 0 , 16, 16)
+
+        pannable = hildon.PannableArea()
+        pannable.add (self.view)
+        pannable.set_size_request_policy(hildon.SIZE_REQUEST_CHILDREN)
+        pannable.show()
+
+        self.align.add(pannable)
+        self.add(self.align)
+        self.align.show()
+
+
+    def _add_button_clicked_cb(self, button, feedsWindow):
+        selection = self.view.get_selection()
+        selected_rows = selection.get_selected_rows()
+        model, paths = selected_rows
+        if not paths:
+            hildon.hildon_banner_show_information(self, '', _('Please select at least one Feed'))
+            return
+
+        for path in paths:
+            url = model[path][FindView.FIND_DATA_COLUMN]
+
+            # Quickly show feedback to user
+            hildon.hildon_gtk_window_set_progress_indicator(feedsWindow, True)
+
+            # Insert a dummy row while information is retrieved
+            dummy_feed_data, feed_iter = feedsWindow.view.add_dummy_feed(url)
+            path = feedsWindow.view.get_model().get_path(feed_iter)
+            row_reference = gtk.TreeRowReference(feedsWindow.view.get_model(), path)
+
+            # Add feed to manager
+            manager.add_feed(url, feedsWindow._feed_added_cb, row_reference)
+
+
+        # restore normal mode
+        self._restore_normal_mode(button)
+
+    def _restore_normal_mode(self, button):
+        self.destroy()
